package com.hikcreate.edl.pub.web.mobile.domain.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hikcreate.edl.pub.web.mobile.domain.IBindService;
import com.hikcreate.edl.pub.web.mobile.infra.core.Result;
import com.hikcreate.edl.pub.web.mobile.infra.core.enums.ResultCode;
import com.hikcreate.edl.pub.web.mobile.infra.data.feign.dcp_service_vehicle.VechicleFeign;
import com.hikcreate.edl.pub.web.mobile.infra.data.feign.dcp_service_vehicle.ViolationFeign;
import com.hikcreate.edl.pub.web.mobile.infra.data.feign.dcp_service_vehicle.param.request.PlateNumAndTypeQueryReq;
import com.hikcreate.edl.pub.web.mobile.infra.data.feign.dcp_service_vehicle.param.request.PlateNumQueryReq;
import com.hikcreate.edl.pub.web.mobile.infra.data.feign.dcp_service_vehicle.param.request.ViolationPlateNumAndTypeQueryReq;
import com.hikcreate.edl.pub.web.mobile.infra.data.feign.dcp_service_vehicle.param.response.VehicleRes;
import com.hikcreate.edl.pub.web.mobile.infra.data.feign.dcp_service_vehicle.param.response.ViolationRes;
import com.hikcreate.edl.pub.web.mobile.infra.data.mapper.BindInfoMapper;
import com.hikcreate.edl.pub.web.mobile.infra.model.BindInfo;
import com.hikcreate.edl.pub.web.mobile.infra.model.BindInfoQuery;
import com.hikcreate.edl.pub.web.mobile.infra.model.param.response.DrivingLicenseInfoRes;
import com.hikcreate.edl.pub.web.mobile.infra.model.param.response.InsuranceInfoRes;
import com.hikcreate.edl.pub.web.mobile.infra.model.param.response.ResultList;
import com.hikcreate.edl.pub.web.mobile.infra.model.param.response.ViolationInfoRes;
import com.hikcreate.edl.pub.web.mobile.infra.model.param.response.YearCheckInfoRes;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

/**
 * description:客户绑定服务实现类
 *
 * @author: xieshixiang
 * @time：2020/6/18 16:12
 **/
@Service
@EnableScheduling
@Slf4j
public class BindServiceImpl extends ServiceImpl<BindInfoMapper, BindInfo> implements IBindService {

    @Resource
    BindInfoMapper mapper;
    @Resource
    VechicleFeign vechicle;
    @Resource
    ViolationFeign violatio;


    @Override
    public Result bind(BindInfo info) {
        info.setUnqId(UUID.randomUUID().toString());
        info.setEditorTime(DateTime.now());

        //判断规则1:用户已绑定的车辆不超过3辆,包括已解绑但未过一个月的车辆
        QueryWrapper boundQuery = new QueryWrapper();
        boundQuery.eq("user_id", info.getUserId());
        Integer bound = mapper.selectCount(boundQuery);
        if (bound >= 3) {
            return Result.errorResult(ResultCode.SERVICE_ERROR);
        }

        //规则2:客户手机号与车辆对应的车主手机号一致
        PlateNumQueryReq req = new PlateNumQueryReq();
        req.setPlateNum(info.getPlateNum());
        List<VehicleRes> byPlateNum = vechicle.getByPlateNum(req);
        for (VehicleRes res : byPlateNum) {
            if (info.getPhone().equals(res.getSjhm())) {
                info.setPlateType(res.getHpzl());
            }
        }
        if (info.getPlateType() == null || info.getPlateType().equals("")) {
            return Result.errorResult(ResultCode.DATA_ERROR);
        }
        return mapper.insert(info) > 0 ? Result.successResult(info.getUnqId()) : Result.errorResult();
    }

    @Override
    public Result<YearCheckInfoRes> yearCheckInfo(BindInfoQuery query) {
        BindInfo bindInfo = mapper.selectById(query.getUnqId());
        if (bindInfo == null || "0".equals(bindInfo.getStatus())) {
            return Result.errorResult(ResultCode.DATA_ERROR);
        }
        PlateNumAndTypeQueryReq req  = new PlateNumAndTypeQueryReq();
        req.setPlateNum(bindInfo.getPlateNum());
        req.setPlateType(bindInfo.getPlateType());
        VehicleRes vehicleRes = vechicle.getByPlateNumAndType(req);
        if( Objects.isNull(vehicleRes) ) {
            return Result.successResult();
        }
        YearCheckInfoRes yearCheckInfoRes = new YearCheckInfoRes();
        yearCheckInfoRes.setYxqz(vehicleRes.getYxqz());
        Long checkTime = DateUtil.parse(vehicleRes.getYxqz(),"yyyy-MM-dd HH:mm:ss").getTime();
        if( checkTime>=System.currentTimeMillis() ) {
            yearCheckInfoRes.setZt("未到期");
        } else {
            yearCheckInfoRes.setZt("已到期");
        }
        return Result.successResult(yearCheckInfoRes);
    }

    @Override
    public Result<InsuranceInfoRes> insuranceInfo(BindInfoQuery query) {
        BindInfo bindInfo = mapper.selectById(query.getUnqId());
        if (bindInfo == null || "0".equals(bindInfo.getStatus())) {
            return Result.errorResult(ResultCode.DATA_ERROR);
        }
        PlateNumAndTypeQueryReq req  = new PlateNumAndTypeQueryReq();
        req.setPlateNum(bindInfo.getPlateNum());
        req.setPlateType(bindInfo.getPlateType());
        VehicleRes vehicleRes = vechicle.getByPlateNumAndType(req);
        if( Objects.isNull(vehicleRes) ) {
            return Result.successResult();
        }
        InsuranceInfoRes insuranceInfoRes = new InsuranceInfoRes();
        if( StrUtil.isNotBlank(vehicleRes.getBxzzrq()) ) {

        } else {

        }
        return null;
    }


    @Override
    public Result<ResultList<ViolationInfoRes>> violationInfo(BindInfoQuery query) {
        BindInfo bindInfo = mapper.selectById(query.getUnqId());
        if (bindInfo == null || "0".equals(bindInfo.getStatus())) {
            return Result.errorResult(ResultCode.DATA_ERROR);
        }

        ViolationPlateNumAndTypeQueryReq req = new ViolationPlateNumAndTypeQueryReq();
        req.setPlateNum(bindInfo.getPlateNum());
        req.setPlateType(bindInfo.getPlateType());
        List<ViolationRes> byPlateNumAndType = violatio.getByPlateNumAndType(req);
        List<ViolationInfoRes> violationInfoResList=new ArrayList<>();
        for (ViolationRes res : byPlateNumAndType){
            ViolationInfoRes info = new ViolationInfoRes();
            BeanUtil.copyProperties(res,info);
            violationInfoResList.add(info);
        }
        return Result.successResult(new ResultList<>(violationInfoResList));
    }

    @Override
    public Result<DrivingLicenseInfoRes> drivingLicenseInfo(BindInfoQuery query) {
        BindInfo bindInfo = mapper.selectById(query.getUnqId());
        if (bindInfo == null || "0".equals(bindInfo.getStatus())) {
            return Result.errorResult(ResultCode.DATA_ERROR);
        }
        return null;

    }


    @Scheduled(cron = "0 0/30 * * * ? ")
    public void clearPast() {
        DateTime now = DateTime.now();
        log.info("开始清除,已过期且无效的绑定信息.当前时间:" + now);
        DateTime lastMonth = DateUtil.offsetDay(now, -30);
        Integer count = mapper.clearPast(lastMonth);
        log.info("清除完毕,共清除过期且无效的绑定信息:" + count);

    }


}
